package graphics;


/**
 * Imports
 */
import java.awt.Color;
import java.awt.DisplayMode;
import java.awt.Font;
import java.awt.FontFormatException;
import java.awt.Frame;
import java.awt.Graphics2D;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Toolkit;
import java.awt.image.BufferStrategy;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import core.Game;


/**
 * Render
 * @author Varkalas D`Lonovan
 */
public class Renderer extends Frame {
	/**
     * Class Variables
     */
	private static final long serialVersionUID = -4720102238516283785L;
	private Game game;
    private GUI activeWindow;
    private TitleGUI titleWindow;
    private CharacterCreationGUI characterCreationWindow;
    private ContinueCharacterGUI continueCharacterWindow;
    private WorldGUI worldWindow;
    private EngineTitleGUI engineTitleWindow;
    private MapMakerGUI mapMakerWindow;
    private ItemMakerGUI itemMakerWindow;
    public static final int TITLE_WINDOW = 0;
    public static final int CHARACTER_CREATION_WINDOW = 1;
    public static final int CONTINUE_CHARACTER_WINDOW = 2;
    public static final int WORLD_WINDOW = 3;
    public static final int ENGINE_TITLE_WINDOW = 4;
    public static final int MAP_MAKER_WINDOW = 5;
    public static final int ITEM_MAKER_WINDOW = 6;
    private BufferStrategy bufferStrategy;
    private long lastTime = 0;
    private Font font;
    private boolean gameRunning = true;
    private boolean engineRunning = false;


    /**
     * Constructor for the Map Maker
     * @param mapMaker The map maker instance.
     */
    /*public Renderer(MapMaker mapMaker) {
    	// TODO
    }*/


    /**
     * Constructor for the Game
     * @param theGame The game instance.
     */
    public Renderer(Game theGame) {
        // setup the window characteristics
        super("Mystic Realms");
        this.setSize(640, 480);
        int screenWidth = Toolkit.getDefaultToolkit().getScreenSize().width;
        int screenHeight = Toolkit.getDefaultToolkit().getScreenSize().height;
        this.setLocation((screenWidth - 640) / 2, (screenHeight - 480) / 2);
        this.setVisible(false);
        this.setResizable(false);
        this.setUndecorated(true);
        this.setVisible(true);

        // let Render know where Game is
        game = theGame;

        // control of the tab key
        this.setFocusTraversalKeysEnabled(false);

        // setup the buffer strategy for rendering
        this.createBufferStrategy(2);
        bufferStrategy = this.getBufferStrategy();

        // load up the fonts
        try {
	        FileInputStream fontFileInputStream = new FileInputStream(new File("Fonts/Abbey-Medium.ttf"));
	        font = (Font.createFont(Font.TRUETYPE_FONT, fontFileInputStream)).deriveFont(12.0f);
        } catch (FileNotFoundException exception) {
        	System.err.println("[Renderer.java]  Unable to locate the font file.");
        	System.err.println(exception);
        	System.exit(-1);
        } catch (FontFormatException exception) {
        	System.err.println("[Renderer.java]  The font file is not in TrueType format.");
        	System.err.println(exception);
        	System.exit(-1);
        } catch (IOException exception) {
        	System.err.println("[Renderer.java]  Unable to read from the font file.");
        	System.err.println(exception);
        	System.exit(-1);
        }

        // load the title screen
        switchWindow(TITLE_WINDOW);
    }


    /**
     * switchWindow
     * @param window The window that gets set to the active window.
     */
    public void switchWindow(int window) {
        // remove the listeners from the current window
        this.removeKeyListener(activeWindow);
        this.removeMouseListener(activeWindow);
        this.removeMouseMotionListener(activeWindow);

        // reset the window size
        this.setSize(640, 480);
        int screenWidth = Toolkit.getDefaultToolkit().getScreenSize().width;
        int screenHeight = Toolkit.getDefaultToolkit().getScreenSize().height;
        this.setLocation((screenWidth - 640) / 2, (screenHeight - 480) / 2);

        // change to the given window
        switch(window) {
            case TITLE_WINDOW:
            	titleWindow = new TitleGUI(this);
                activeWindow = titleWindow;
                break;
            case CHARACTER_CREATION_WINDOW:
            	characterCreationWindow = new CharacterCreationGUI(this);
            	activeWindow = characterCreationWindow;
            	break;
            case CONTINUE_CHARACTER_WINDOW:
            	continueCharacterWindow = new ContinueCharacterGUI(this);
            	activeWindow = continueCharacterWindow;
            	break;
            case WORLD_WINDOW:
            	worldWindow = new WorldGUI(this);
            	activeWindow = worldWindow;
            	break;
            case ENGINE_TITLE_WINDOW:
            	engineTitleWindow = new EngineTitleGUI(this);
            	activeWindow = engineTitleWindow;
            	break;
            case MAP_MAKER_WINDOW:
            	mapMakerWindow = new MapMakerGUI(this);
            	activeWindow = mapMakerWindow;
            	break;
            case ITEM_MAKER_WINDOW:
            	itemMakerWindow = new ItemMakerGUI(this);
            	activeWindow = itemMakerWindow;
            	break;
            default:
            	break;
        }

        // add the listeners to the given window
        this.addKeyListener(activeWindow);
        this.addMouseListener(activeWindow);
        this.addMouseMotionListener(activeWindow);
    }


    /**
     * gameLoop
     */
    public void gameLoop() {
    	// get the graphics device for fullscreen & original information
    	GraphicsEnvironment graphicsEnvironment = GraphicsEnvironment.getLocalGraphicsEnvironment();
        GraphicsDevice graphicsDevice = graphicsEnvironment.getDefaultScreenDevice();
        DisplayMode originalDisplayMode = graphicsDevice.getDisplayMode();

        // get the starting time
        long lastLoopTime = System.currentTimeMillis();

        try {
        	// set to fullscreen
        	engineRunning = true;  // TODO: change to false after testing
        	if (!engineRunning) {
	        	graphicsDevice.setFullScreenWindow(this);
	        	graphicsDevice.setDisplayMode(new DisplayMode(640, 480, 32, 60));
        	}

        	// start the loop
	        while (gameRunning) {
	        	// handle time
	        	long delta = System.currentTimeMillis() - lastLoopTime;
	            lastLoopTime = System.currentTimeMillis();

	        	// create a graphics context for the drawing buffer
	        	Graphics2D graphics = (Graphics2D) bufferStrategy.getDrawGraphics();
	        	graphics.setColor(Color.black);
	        	graphics.fillRect(0, 0, 640, 480);
	        	graphics.setFont(font);

	        	// draw the correct window
	        	activeWindow.timeChanges(delta);
	        	activeWindow.render(graphics);

	        	// disposes of the graphics context
	        	graphics.dispose();

	        	// make the next available buffer visible
	        	bufferStrategy.show();

	        	// make sure the game runs smoothly
	        	long currentTime = System.currentTimeMillis();
	        	if (currentTime - lastTime < 20) {
	        		try {
	        			Thread.sleep(20 - (currentTime - lastTime));
	        		} catch (InterruptedException e) {
	        			System.err.println("[Renderer.java]  Renderer.java: Thread sleep problem.");
	        			System.err.println(e);
	        		}
	            }
	        	lastTime = currentTime;
	        }
        } finally {
        	// return to original settings
        	if (!engineRunning) {
	        	graphicsDevice.setDisplayMode(originalDisplayMode);
	        	graphicsDevice.setFullScreenWindow(null);
        	}
        }

        System.exit(0);
    }


    /**
     * setGameRunning
     * @param isGameRunning Sets whether the game is running or not.
     */
    public void setGameRunning(boolean isGameRunning) {
    	gameRunning = isGameRunning;
    }


    /**
     * setEngineRunning
     * @param isEngineRunning Sets whether the MapMaker is running or not.
     */
    public void setEngineRunning(boolean isEngineRunning) {
    	engineRunning = isEngineRunning;
    }


    /**
	 * @return Returns the game.
	 */
	public Game getGame() {
		return game;
	}
}